//------------------------------------------------------------------//
//------------------------------------------------------------------//
//- INCLUDES -------------------------------------------------------//
//------------------------------------------------------------------//
//------------------------------------------------------------------//

#define WIN32_LEAN_AND_MEAN
#define INIT_GUID

#include <windows.h>		//Windows header file
#include <windowsx.h>		//Windows header with some kewl extras
#include <mmsystem.h>
#include <iostream.h>			
#include <conio.h>
#include <stdio.h> 
#include <stdarg.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <math.h>
#include <io.h>
#include <fcntl.h>
#include <time.h>

#include <dinput.h>

#include <gl\gl.h>			//OpenGL header file
#include <gl\glu.h>			//The OpenGL Glu header file

#pragma comment(lib, "Winmm.lib")	//Search for the windows multimedia library at compile time
#pragma comment(lib, "Dxguid.lib")	//Search for the DXguid library at compile time
#pragma comment(lib, "Dinput8.lib")	//Search for the DirectInput8 library at compile time
#pragma comment(lib, "OpenGL32.lib")//Search for the OpenGL32 library at compile time
#pragma comment(lib, "Glu32.lib")	//Search for the Glu32 library at compile time


//------------------------------------------------------------------//
//------------------------------------------------------------------//
//- CONSTANTS ------------------------------------------------------//
//------------------------------------------------------------------//
//------------------------------------------------------------------//
#define CLASSNAME "ShiningKnight"


/* Window type constants (used with the fullscreen flag) */
#define FULLSCREEN  0
#define WINDOW		1
#define GIVE_CHOICE 2

/* Fullscreen flags */
#define FULLSCREEN_ON  0
#define FULLSCREEN_OFF 1

/* Mouse constants */
#define MOUSE_LEFT   0
#define MOUSE_RIGHT  1
#define MOUSE_MIDDLE 2

//------------------------------------------------------------------//
//------------------------------------------------------------------//
//- CLASSES --------------------------------------------------------//
//------------------------------------------------------------------//
//------------------------------------------------------------------//

//------------------------------------------------------------------//
//- class SHINING3D ------------------------------------------------//
//------------------------------------------------------------------//
//- The main graphics/window class.  Has functions for creating a  -//
//- window, and various other stuff.							   -//
//------------------------------------------------------------------//
class SHINING3D
	{
	private:
		HDC   hdc;					//Window device context
		HGLRC hrc;					//OGL Rendering context

		unsigned int base;

	public: 
		int SCREEN_WIDTH;
		int SCREEN_HEIGHT;
		int SCREEN_BPP;
		bool fullscreen;
		char* title;


	/* Windows functions */
	bool HandleMessages(void);
	void OpenGLShutdown(void);
	bool OpenGLInit(int width, int height, int bpp, int screenflag, char* Title);
	inline void BufferSwap(void)
		{	SwapBuffers(hdc);	}

	/* Font functions (PUBLIC) */
	void StartTextMode(void);
	void EndTextMode(void);
	void FontInit(void);
	void FontShutdown(void);
	void glPrint(int x, int y, const char *string, ...);
	
	SHINING3D()
		{	}
	~SHINING3D()
		{	}
	};


//------------------------------------------------------------------//
//- class SHININGINPUT ---------------------------------------------//
//------------------------------------------------------------------//
//- The input class, handles all input under DirectInput.  Use the -//
//- KEY_DOWN macro to see if a key is down or not.				   -//
//------------------------------------------------------------------//
extern unsigned char keyBuffer[256];//The keyboard's key buffer
//------------------------------------------------------------------//
class SHININGINPUT
{
	private:
		LPDIRECTINPUT8       lpdi;			//Main DirectInput object
		LPDIRECTINPUTDEVICE8 lpdiKeyboard;	//Keyboard's DirectInput device
		LPDIRECTINPUTDEVICE8 lpdiMouse;		//Mouse's DirectInput device


		void GetWheelMovement(void);
		void GetMouseMovement(void);

	public:
		int x;		//Mouse positions
		int y;
		int z;		//For wheel movement

		bool Init(void);
		void Shutdown(void);
		void Update(void);

		SHININGINPUT()
		{ memset(&keyBuffer, 0, sizeof(unsigned char[256])); }
		~SHININGINPUT()
		{ }
};

//------------------------------------------------------------------//
//------------------------------------------------------------------//
//- GLOBALS --------------------------------------------------------//
//------------------------------------------------------------------//
//------------------------------------------------------------------//
extern HINSTANCE hinstance; //Handle to the window's instance
extern HWND  hwnd;			//Handle to the actual window

extern DIMOUSESTATE  mouseState;	//The mouse's 'buffer'

extern RECT window;			//Used for entering text mode

//------------------------------------------------------------------//
//------------------------------------------------------------------//
//- MACROS ---------------------------------------------------------//
//------------------------------------------------------------------//
//------------------------------------------------------------------//
/*	     This is one of the coolest macros!  It tests to see if a   */
/*       key is currently pressed down or not.  Uses the DirectInput*/
/*       DIK_ key constants.                                        */
#define KEY_DOWN(key) (keyBuffer[key] & 0x80)
#define MOUSE_DOWN(button) (mouseState.rgbButtons[button] & 0x80)

//------------------------------------------------------------------//
//------------------------------------------------------------------//
//- FUNCTION DECLARATIONS ------------------------------------------//
//------------------------------------------------------------------//
//------------------------------------------------------------------//
extern LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);
extern void ResizeGLScene(GLsizei width, GLsizei height);

